#include "mapviewer.h"
#include "functions.h"


std::vector<QCPCurve *> lane_lines;

void MapViewer::Init(QCustomPlot *customPlot_, QCustomPlot *_customPlot2)
{
    customPlot = customPlot_;
    customPlot_2 = _customPlot2;

    SetHostCorners();
    SetBackGround(customPlot);
    SetBackGround(customPlot_2);

    QPen pen;
    pen.setColor(QColor("#FFFAFA"));
    pen.setWidth(5);
    pen.setStyle(Qt::PenStyle::SolidLine);

    odo_curve = new QCPCurve(customPlot_2->xAxis, customPlot_2->yAxis);
    odo_curve->setPen(pen);


    center_line = new QCPCurve(customPlot_2->xAxis, customPlot_2->yAxis);

    odo_focused = true;

    lane_lines.push_back(new QCPCurve(customPlot->xAxis, customPlot->yAxis));
    lane_lines.push_back(new QCPCurve(customPlot->xAxis, customPlot->yAxis));
    lane_lines.push_back(new QCPCurve(customPlot->xAxis, customPlot->yAxis));

}

void MapViewer::SetHostCorners()
{
    //vehicle length(mm)
    float v_length = 4840.6/1000.0f;
    float v_width = 1935/1000.0f;
    //float v_height = 1730/1000.0f;

    //axes length
    float v_axes_length = 2920/1000.0f;
    //front suspension
    //float v_fs = 956/1000.0f;
    float v_rs = 995.2/1000.0f;

    Eigen::Vector2d r_l_p = Eigen::Vector2d(v_width/2, -v_rs);
    Eigen::Vector2d r_r_p = Eigen::Vector2d(-v_width/2, -v_rs);
    Eigen::Vector2d f_l_p = Eigen::Vector2d(v_width/2, v_length - v_rs);
    Eigen::Vector2d f_r_p = Eigen::Vector2d(-v_width/2, v_length - v_rs);

    Eigen::Vector2d r_l_w = Eigen::Vector2d(v_width/2, 0);
    Eigen::Vector2d r_r_w = Eigen::Vector2d(-v_width/2, 0);
    Eigen::Vector2d f_l_w = Eigen::Vector2d(v_width/2, v_axes_length);
    Eigen::Vector2d f_r_w = Eigen::Vector2d(-v_width/2, v_axes_length);

    m_host_corner.push_back(r_l_p);
    m_host_corner.push_back(f_l_p);
    m_host_corner.push_back(f_r_p);
    m_host_corner.push_back(r_r_p);

    m_host_corner.push_back(r_l_w);
    m_host_corner.push_back(r_r_w);
    m_host_corner.push_back(f_l_w);
    m_host_corner.push_back(f_r_w);

    vehicle_shp.clear();

    vehicle_shp.insert(pair<int, QCPCurve *>(0, new QCPCurve(customPlot_2->xAxis, customPlot_2->yAxis)));
    vehicle_shp.insert(pair<int, QCPCurve *>(1, new QCPCurve(customPlot_2->xAxis, customPlot_2->yAxis)));


    QPen pen0;
    pen0.setColor(QColor("#1E90FF"));
    pen0.setWidth(5);
    pen0.setStyle(Qt::PenStyle::SolidLine);

    vehicle_shp[0]->setPen(pen0);
    vehicle_shp[0]->setBrush(QBrush(QColor(255,192,203,50)));

    QPen pen1;
    pen1.setColor(QColor("#FF6347"));
    pen1.setWidth(4);
    pen1.setStyle(Qt::PenStyle::DashLine);

    vehicle_shp[1]->setPen(pen1);
}

void MapViewer::plotOdo(double x, double y, double heading)
{

    static Eigen::Vector2d last_odo(-0.01,0);

    Eigen::Vector2d cur_odo(x, y);

    if((cur_odo-last_odo).norm() > 10)
    {
        odo_curve->data().data()->clear();
    }

    if(cur_odo == last_odo)return;

    last_odo = cur_odo;

    odo_curve->addData(cur_odo.x(),cur_odo.y());


    std::lock_guard<std::mutex> lock(mtx_ui);
    if(odo_focused)
    {
        customPlot_2->xAxis->setRange(x, customPlot_2->width()/10, Qt::AlignCenter);
        customPlot_2->yAxis->setRange(y, customPlot_2->height()/10, Qt::AlignCenter);
    }


    vector<Eigen::Vector2d> _nps;

    for (auto _p:m_host_corner) {

        _nps.push_back(Functions::vcs2gcs(cur_odo, -(heading), _p));
    }

    vehicle_shp[0]->data().data()->clear();
    vehicle_shp[1]->data().data()->clear();

    vehicle_shp[0]->addData(_nps[0].x(), _nps[0].y());
    vehicle_shp[0]->addData(_nps[1].x(), _nps[1].y());
    vehicle_shp[0]->addData(_nps[2].x(), _nps[2].y());
    vehicle_shp[0]->addData(_nps[3].x(), _nps[3].y());
    vehicle_shp[0]->addData(_nps[0].x(), _nps[0].y());


    vehicle_shp[1]->addData(_nps[4].x(), _nps[4].y());
    vehicle_shp[1]->addData(_nps[5].x(), _nps[5].y());
    vehicle_shp[1]->addData((_nps[4].x() + _nps[5].x())/2, (_nps[4].y()+_nps[5].y())/2);
    vehicle_shp[1]->addData((_nps[6].x() + _nps[7].x())/2, (_nps[6].y()+_nps[7].y())/2);
    vehicle_shp[1]->addData(_nps[6].x(), _nps[6].y());
    vehicle_shp[1]->addData(_nps[7].x(), _nps[7].y());

    try {
        customPlot_2->replot();
    } catch (unexpected_handler e) {

    }

}

void MapViewer::Show3DQuaternion(QWidget* _p)
{
    if(!chartForm_flag)
    {
        chartForm = new ChartForm(_p);
        chartForm->setAttribute(Qt::WA_DeleteOnClose); // 确保关闭时删除对象
//        chartForm->setWindowTitle("3D Quaternion"); // 设置窗口标题
//        chartForm->resize(800, 600); // 设置窗口大小
        chartForm->show(); // 显示窗口

        chartForm->NewLine(1, Qt::red, 2);
        chartForm->NewLine(2, Qt::yellow, 2);
        chartForm->NewLine(3, Qt::green, 2);
        chartForm->NewLine(4, Qt::blue, 2);

        chartForm_flag = true;
    }else
    {
        chartForm->close();
        chartForm_flag = false;
    }
}


void MapViewer::SetBackGround(QCustomPlot * customPlot)
{
    customPlot->xAxis->setBasePen(QPen(Qt::white, 1));
    customPlot->yAxis->setBasePen(QPen(Qt::white, 1));
    customPlot->xAxis->setTickPen(QPen(Qt::white, 1));
    customPlot->yAxis->setTickPen(QPen(Qt::white, 1));
    customPlot->xAxis->setSubTickPen(QPen(Qt::white, 1));
    customPlot->yAxis->setSubTickPen(QPen(Qt::white, 1));
    customPlot->xAxis->setTickLabelColor(Qt::white);
    customPlot->yAxis->setTickLabelColor(Qt::white);
    customPlot->xAxis->grid()->setPen(QPen(QColor(140, 140, 140), 1, Qt::DotLine));
    customPlot->yAxis->grid()->setPen(QPen(QColor(140, 140, 140), 1, Qt::DotLine));
    customPlot->xAxis->grid()->setSubGridPen(QPen(QColor(80, 80, 80), 1, Qt::DotLine));
    customPlot->yAxis->grid()->setSubGridPen(QPen(QColor(80, 80, 80), 1, Qt::DotLine));
    customPlot->xAxis->grid()->setSubGridVisible(true);
    customPlot->yAxis->grid()->setSubGridVisible(true);
    customPlot->xAxis->grid()->setZeroLinePen(Qt::NoPen);
    customPlot->yAxis->grid()->setZeroLinePen(Qt::NoPen);
    customPlot->xAxis->setUpperEnding(QCPLineEnding::esSpikeArrow);
    customPlot->yAxis->setUpperEnding(QCPLineEnding::esSpikeArrow);
    QLinearGradient plotGradient;
    plotGradient.setStart(0, 0);
    plotGradient.setFinalStop(0, 350);
    plotGradient.setColorAt(0, QColor(80, 80, 80));
    plotGradient.setColorAt(1, QColor(50, 50, 50));
    customPlot->setBackground(plotGradient);
    QLinearGradient axisRectGradient;
    axisRectGradient.setStart(0, 0);
    axisRectGradient.setFinalStop(0, 350);
    axisRectGradient.setColorAt(0, QColor(80, 80, 80));
    axisRectGradient.setColorAt(1, QColor(130, 130, 130));
    customPlot->axisRect()->setBackground(axisRectGradient);

    customPlot->setInteractions(QCP::iRangeDrag | QCP::iRangeZoom);
    customPlot->setOpenGl(true);
    qDebug()<< "“opengle=”" << customPlot->openGl();
}



void MapViewer::PlotLine(QVector<QCPCurveData> dataSpiral, QColor _color, int _width)
{
    QCPCurve * odo_line;

    odo_line = new QCPCurve(customPlot_2->xAxis, customPlot_2->yAxis);

    // pass the data to the curves; we know t (i in loop above) is ascending, so set alreadySorted=true (saves an extra internal sort):
    odo_line->data()->set(dataSpiral, true);

    QPen pen;
    pen.setWidth(_width);
    pen.setColor(_color);

    odo_line->setPen(pen);

    //customPlot->rescaleAxes();
    //customPlot->replot();
}

void MapViewer::PlotLine_c(QVector<QCPCurveData> dataSpiral, QColor _color, int _width)
{

    // pass the data to the curves; we know t (i in loop above) is ascending, so set alreadySorted=true (saves an extra internal sort):
    center_line->data()->set(dataSpiral, true);

    QPen pen;
    pen.setWidth(_width);
    pen.setColor(_color);

    center_line->setPen(pen);
    //customPlot->rescaleAxes();
    //customPlot->replot();
}

void MapViewer::PlotLines(QVector<QCPCurveData> dataSpiral, QColor _color, int _width, int id)
{
    auto odo_line{lane_lines[id]};

    // pass the data to the curves; we know t (i in loop above) is ascending, so set alreadySorted=true (saves an extra internal sort):
    odo_line->data()->set(dataSpiral, true);

    QPen pen;
    pen.setWidth(_width);
    pen.setColor(_color);

    odo_line->setPen(pen);

    //customPlot->rescaleAxes();
    customPlot->replot();
}

void MapViewer::PlotPoints(QVector<QCPCurveData> dataSpiral, QColor _color)
{
    // 绘制散点
    QCPGraph * curGraph = customPlot_2->addGraph();

    QPen pen;
    pen.setWidth(5);
    pen.setColor(_color);

    curGraph->setPen(pen);
    curGraph->setLineStyle(QCPGraph::lsNone);
    curGraph->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssCircle, 2));

    QVector<double> latVector(dataSpiral.size());
    QVector<double> lonVector(dataSpiral.size());

    for (int i = 0; i < dataSpiral.size(); ++i) {

        latVector[i] = dataSpiral[i].key;
        lonVector[i] = dataSpiral[i].value;
    }

    curGraph->setData(latVector, lonVector);

    //customPlot->rescaleAxes();
    //customPlot->replot();
}

//void MapViewer::SetHostCorners()
//{
//    //vehicle length(mm)
//    float v_length = 4840.6/1000.0f;
//    float v_width = 1935/1000.0f;
//    //float v_height = 1730/1000.0f;

//    //axes length
//    float v_axes_length = 2920/1000.0f;
//    //front suspension
//    //float v_fs = 956/1000.0f;
//    float v_rs = 995.2/1000.0f;

//    Eigen::Vector2d r_l_p = Eigen::Vector2d(v_width/2, -v_rs);
//    Eigen::Vector2d r_r_p = Eigen::Vector2d(-v_width/2, -v_rs);
//    Eigen::Vector2d f_l_p = Eigen::Vector2d(v_width/2, v_length - v_rs);
//    Eigen::Vector2d f_r_p = Eigen::Vector2d(-v_width/2, v_length - v_rs);

//    Eigen::Vector2d r_l_w = Eigen::Vector2d(v_width/2, 0);
//    Eigen::Vector2d r_r_w = Eigen::Vector2d(-v_width/2, 0);
//    Eigen::Vector2d f_l_w = Eigen::Vector2d(v_width/2, v_axes_length);
//    Eigen::Vector2d f_r_w = Eigen::Vector2d(-v_width/2, v_axes_length);

//    m_host_corner.push_back(r_l_p);
//    m_host_corner.push_back(f_l_p);
//    m_host_corner.push_back(f_r_p);
//    m_host_corner.push_back(r_r_p);

//    m_host_corner.push_back(r_l_w);
//    m_host_corner.push_back(r_r_w);
//    m_host_corner.push_back(f_l_w);
//    m_host_corner.push_back(f_r_w);

//    vehicle_shp.clear();

//    vehicle_shp.insert(pair<int, QCPCurve *>(0, new QCPCurve(customPlot->xAxis, customPlot->yAxis)));
//    vehicle_shp.insert(pair<int, QCPCurve *>(1, new QCPCurve(customPlot->xAxis, customPlot->yAxis)));


//    QPen pen0;
//    pen0.setColor(QColor("#1E90FF"));
//    pen0.setWidth(5);
//    pen0.setStyle(Qt::PenStyle::SolidLine);

//    vehicle_shp[0]->setPen(pen0);
//    vehicle_shp[0]->setBrush(QBrush(QColor(255,192,203,50)));

//    QPen pen1;
//    pen1.setColor(QColor("#FF6347"));
//    pen1.setWidth(4);
//    pen1.setStyle(Qt::PenStyle::DashLine);

//    vehicle_shp[1]->setPen(pen1);
//}
